from __future__ import annotations


class ValueUnassignableToDescription(Exception):
    'Raised when a description field has unacceptable value'
    # example
    # length: 5.2
    pass


class DescriptionFileNotLoading(Exception):
    pass


class FileHasNoExtension(Exception):
    pass


class EnvironmentFileNotLoading(Exception):
    pass


class RequiredVariableNotSet(Exception):
    pass


class CallBackNotRunning(Exception):
    pass



from __future__ import annotations

import unittest

from src.env_should_be.description import *
from src.env_should_be.exception import *
from src.env_should_be.utils import *


class TestLength(unittest.TestCase):
    def setUp(self):
        self.cls = Length
        super().setUp()

    def test_is_valid(self):
        for v in [4, 5, 2, 6, 8]:
            instance = self.cls(v)
            self.assertTrue(instance.is_valid(v))

        for v in ['E', None, 1.2, 0]:
            self.assertRaises(ValueUnassignableToDescription, self.cls, v)

    def test_does_pass(self):
        for expected, value in [(4, 'aabb'), (1, 'a'), (10, 'aaabbbccce')]:
            instance = self.cls(expected)
            self.assertTrue(instance.does_pass(value))

        for expected, value in [(4, {}), (1, 1.2), (2, None)]:
            instance = self.cls(expected)
            self.assertFalse(instance.does_pass(value))


class TestMinLength(unittest.TestCase):
    def setUp(self):
        self.cls = MinLength
        super().setUp()

    def test_is_valid(self):
        for v in [4, 5, 2, 6, 8]:
            instance = self.cls(v)
            self.assertTrue(instance.is_valid(v))

        for v in ['E', None, 1.2, 0]:
            self.assertRaises(ValueUnassignableToDescription, self.cls, v)

    def test_does_pass(self):
        for expected, value in [(4, 'aabb4'), (1, 'a11'), (10, 'aaabbbccce111')]:
            instance = self.cls(expected)
            self.assertTrue(instance.does_pass(value))

        for expected, value in [(4, 'aab'), (4, {}), (1, 1.2), (2, None)]:
            instance = self.cls(expected)
            self.assertFalse(instance.does_pass(value))

    def test_get_name(self):
        self.assertEqual(to_snake_case(self.cls.__name__), 'min_length')


class TestMaxLength(unittest.TestCase):
    def setUp(self):
        self.cls = MaxLength
        super().setUp()

    def test_is_valid(self):
        for v in [4, 5, 2, 6, 8]:
            instance = self.cls(v)
            self.assertTrue(instance.is_valid(v))

        for v in ['E', None, 1.2, 0]:
            self.assertRaises(ValueUnassignableToDescription, self.cls, v)

    def test_does_pass(self):
        for expected, value in [(4, 'aabb'), (2, 'a'), (2, ''), (4, '123')]:
            instance = self.cls(expected)
            self.assertTrue(instance.does_pass(value))

        for expected, value in [(4, 'aabb5'), (4, {}), (1, 1.2), (2, None)]:
            instance = self.cls(expected)
            self.assertFalse(instance.does_pass(value))

    def test_get_name(self):
        self.assertEqual(to_snake_case(self.cls.__name__), 'max_length')


class TestOption(unittest.TestCase):
    def setUp(self):
        self.cls = Option
        super().setUp()

    def test_is_valid(self):
        for v in [['d', 5], [2.2, False]]:
            instance = self.cls(v)
            self.assertTrue(instance.is_valid(v))

        for v in ['E', None, 1.2, 0, [], {}]:
            self.assertRaises(ValueUnassignableToDescription, self.cls, v)

    def test_does_pass(self):
        for expected, value in [
            (['dev', 'prod'], 'prod'),
            (['0.0.0.0', 'localhost'], '0.0.0.0'),
            ([5000, 6000], 5000),
        ]:
            instance = self.cls(expected)
            self.assertTrue(instance.does_pass(value))

        for expected, value in [
            (['dev', 'prod'], 'local'),
            (['0.0.0.0', 'localhost'], 'http://127.0.0.1'),
            ([5000, 6000], 7894),
        ]:
            instance = self.cls(expected)
            self.assertFalse(instance.does_pass(value))

    def test_get_name(self):
        self.assertEqual(to_snake_case(self.cls.__name__), 'option')


class TestRegex(unittest.TestCase):
    def setUp(self):
        self.cls = Regex
        self.valid_regex = [
            r'(?i)(\W|^)(baloney|darn|drat|fooey|gosh\sdarnit|heck)(\W|$)',
            r'(\W|^)[\w.\-]{0,25}@(yahoo|hotmail|gmail)\.com(\W|$)',
            r'192\.168\.1\.\d{1,3}$',
        ]
        super().setUp()

    def test_is_valid(self):
        for v in self.valid_regex:
            instance = self.cls(v)
            self.assertTrue(instance.is_valid(v))

        for v in [False, {}, None, 1.2, 0]:
            self.assertRaises(ValueUnassignableToDescription, self.cls, v)

    def test_does_pass(self):
        for expected, value in [
            (self.valid_regex[0], 'baloney'),
            (self.valid_regex[1], 'wassef@yahoo.com'),
            (self.valid_regex[2], '192.168.1.255'),
        ]:
            instance = self.cls(expected)
            self.assertTrue(instance.does_pass(value))

        for expected, value in [
            (self.valid_regex[0], 'stock dips'),
            (self.valid_regex[1], 'wassef@company.tn'),
            (self.valid_regex[2], '192.168.1.5555'),
        ]:
            instance = self.cls(expected)
            self.assertFalse(instance.does_pass(value))

    def test_get_name(self):
        self.assertEqual(to_snake_case(self.cls.__name__), 'regex')


class TestConstant(unittest.TestCase):
    def setUp(self):
        self.cls = Constant
        super().setUp()

    def test_is_valid(self):
        for v in [
            'True',
            'False',
            'localhost',
            'domain.com.org',
            {},
            '$#{}',
            5555,
            1.2,
            None,
            'null',
        ]:
            instance = self.cls(v)
            self.assertTrue(instance.is_valid(v))

    def test_does_pass(self):
        for expected, value in [
            (5, '5'),
            ('a string', 'a string'),
            (10.2, '10.2'),
            (True, 'True'),
        ]:
            instance = self.cls(expected)
            self.assertTrue(instance.does_pass(value))

        for expected, value in [(4, 'aab'), (4, {}), (1, 1.2), (2, None)]:
            instance = self.cls(expected)
            self.assertFalse(instance.does_pass(value))

    def test_get_name(self):
        self.assertEqual(to_snake_case(self.cls.__name__), 'constant')


class TestIsInt(unittest.TestCase):
    def setUp(self):
        self.cls = IsInt
        super().setUp()

    def test_is_valid(self):
        for v in [True, False]:
            instance: Boolean = self.cls(v)
            self.assertTrue(instance.is_valid(v))

        for v in ['E', None, 1.2, 0, 'True', 'False']:
            self.assertRaises(ValueUnassignableToDescription, self.cls, v)

    def test_does_pass(self):
        for expected, value in [(True, 21), (False, 'AbC')]:
            instance: Boolean = self.cls(expected)
            self.assertTrue(instance.does_pass(value))

        for expected, value in [(True, {}), (True, 1.2), (True, None), (False, False)]:
            instance: Boolean = self.cls(expected)
            self.assertFalse(instance.does_pass(value))

    def test_get_name(self):
        self.assertEqual(to_snake_case(self.cls.__name__), 'is_int')


class TestIsFloat(unittest.TestCase):
    def setUp(self):
        self.cls = IsFloat
        super().setUp()

    def test_is_valid(self):
        for v in [True, False]:
            instance: Boolean = self.cls(v)
            self.assertTrue(instance.is_valid(v))

        for v in ['E', None, 1.2, 0, 'True', 'False', {}, []]:
            self.assertRaises(ValueUnassignableToDescription, self.cls, v)

    def test_does_pass(self):
        for expected, value in [(True, 2.2), (False, '21.2'), (False, 2)]:
            instance: Boolean = self.cls(expected)
            self.assertTrue(instance.does_pass(value))

        for expected, value in [(True, {}), (True, 1), (True, None), (True, False)]:
            instance: Boolean = self.cls(expected)
            self.assertFalse(instance.does_pass(value))

    def test_get_name(self):
        self.assertEqual(to_snake_case(self.cls.__name__), 'is_float')


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

import json
import unittest

from yaml.scanner import ScannerError

from src.env_should_be.utils import *


class TestEnvFileFunctions(unittest.TestCase):
    def test_load_env_file(self):
        # Test with a valid file
        file_path = 'test_file.env'
        with open(file_path, 'w') as file:
            file.write('TEST_KEY=TEST_VALUE')
        env_vars = load_env_file(file_path)
        self.assertEqual(env_vars, {'TEST_KEY': 'TEST_VALUE'})
        os.remove(file_path)

        # Test with an invalid file
        file_path = 'invalid_file.env'
        self.assertRaises(FileNotFoundError, load_env_file, file_path)

    def test_load_all_env_vars(self):
        env_vars = load_all_env_vars()
        self.assertIsInstance(env_vars, dict)

    def test_load_json_file(self):
        # Test with a valid file
        file_path = 'test_file.json'
        with open(file_path, 'w') as file:
            file.write('{"TEST_KEY": "TEST_VALUE"}')
        json_data = load_json_file(file_path)
        self.assertEqual(json_data, {'TEST_KEY': 'TEST_VALUE'})
        os.remove(file_path)

        # Test with an invalid file
        file_path = 'invalid_file.json'
        self.assertRaises(FileNotFoundError, load_json_file, file_path)


class TestGetErrorsForEnv(unittest.TestCase):
    test_file_path = 'description.json'

    def tearDown(self):
        if os.path.exists(self.test_file_path):
            os.remove(self.test_file_path)

    def create_json_file(self, data, file_path):
        with open(file_path, 'w') as json_file:
            json.dump(data, json_file)

    def destroy_json_file(self, file_path):
        if os.path.exists(file_path):
            os.remove(file_path)

    def test_load_env_file(self):
        env = {'DB_USER': 'alfred_computer_man', 'DB_PORT': 5432}
        self.create_json_file(
            {
                'DB_USER': {'length': 19, 'required': True},
                'DB_PORT': {'option': [5432, 5431], 'constant': 5432, 'is_int': True},
            },
            self.test_file_path,
        )
        actual = get_errors_for(env, [self.test_file_path])
        self.assertEqual(actual, [])

        actual = get_errors_for(
            {**env, 'DB_PORT': 8000}, [self.test_file_path])
        self.assertTrue(actual.__len__(), 1)

        file_path = 'invalid_file.something'
        self.assertRaises(FileHasNoExtension, get_errors_for, env, [file_path])


class TestGetFileExtension(unittest.TestCase):
    def test_json_extension(self):
        file_path = 'example.json'
        result = get_file_extension(file_path)
        self.assertEqual(result, '.json')

    def test_yaml_extension(self):
        file_path = 'data.yaml'
        result = get_file_extension(file_path)
        self.assertEqual(result, '.yaml')

    def test_yml_extension(self):
        file_path = 'config.yml'
        result = get_file_extension(file_path)
        self.assertEqual(result, '.yml')

    def test_no_extension(self):
        file_path = 'myfile'
        result = get_file_extension(file_path)
        self.assertEqual(result, '')

    def test_mixed_case_extension(self):
        file_path = 'Docume,nt..JSON'
        result = get_file_extension(file_path)
        self.assertEqual(result, '.json')


class TestLoadYAMLFile(unittest.TestCase):
    test_file_path = 'afile.yaml'

    def tearDown(self):
        if os.path.exists(self.test_file_path):
            os.remove(self.test_file_path)

    def test_valid_yaml_file(self):
        test_yaml_content = """
        key1: value1
        key2: value2
        """
        with open('test.yaml', 'w') as test_file:
            test_file.write(test_yaml_content)

        try:
            result = load_yaml_file('test.yaml')
            expected_data = safe_load(test_yaml_content)
            self.assertEqual(result, expected_data)
        finally:
            os.remove('test.yaml')  # Clean up the temporary test file

    def test_nonexistent_file(self):
        with self.assertRaises(FileNotFoundError) as context:
            load_yaml_file('nonexistent.yaml')

        self.assertTrue(
            'nonexistent.yaml does not exist.' in str(context.exception))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
from __future__ import annotations

import argparse
import logging
import subprocess
from json.decoder import JSONDecodeError

from .exception import CallBackNotRunning
from .exception import EnvironmentFileNotLoading
from .utils import *

logging.basicConfig(
    level=logging.INFO,  # Adjust the log level as needed
    format='%(levelname)s: %(message)s',
)


class Namespace:
    description: List[str]
    fail_silently: bool
    env_file: bool
    callback: str


arguments = [
    {
        'dest': 'description',
        'option_strings': ['-d', '--description'],
        'nargs': '+',
        'help': '<Required> either one or multiple paths for description files.',
        'required': True,
    },
    {
        'dest': 'fail_silently',
        'option_strings': ['-fs', '--fail-silently'],
        'type': bool,
        'help': '<Optional> will return an exit status of 0 even if the description(s) fail to match the current env (still triggers the callback).',
        'required': False,
        'default': False,
    },
    {
        'dest': 'env_file',
        'option_strings': ['-e', '--env-file'],
        'type': str,
        'help': '<Optional> not specifying a path to a specific env file to validate description(s) against, environment variables in the current shell will be loaded instead.',
        'required': False,
        'default': None,
    },
    {
        'dest': 'callback',
        'option_strings': ['-cb', '--callback'],
        'type': str,
        'help': '<Optional> not specifying a path to a specific env file to validate description(s) against, environment variables in the current shell will be loaded instead.',
        'required': False,
        'default': None,
    },
]

if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='How should your environment be?')
    for arg in arguments:
        parser.add_argument(*arg['option_strings'], **arg)
    args: Namespace = parser.parse_args()
    if args.env_file:
        try:
            env = load_env_file(args.env_file)
        except (
            FileNotFoundError,
            ValueError,
            JSONDecodeError,
        ) as exc:
            raise EnvironmentFileNotLoading(
                f"couldn't load file at:{args.env_file}, {exc}"
            )
    else:
        env = load_all_env_vars()
    errors = get_errors_for(env, args.description)
    if errors.__len__() > 0:
        for e in errors:
            logging.warning(f'Env Not matching {e.description_path}')
            for variable, fails in e.errors:
                logging.error(f'\n {variable}, failing to match {fails}')
        if args.callback != None:
            try:
                subprocess.run([args.callback])
            except Exception as exc:
                raise CallBackNotRunning(
                    f"couldn't the callback script: {args.callback}, {exc}"
                )
        if not args.fail_silently:
            exit(1)
    exit(0)



from __future__ import annotations

import inspect
import os
import re
from dataclasses import dataclass
from dataclasses import field
from json import JSONDecodeError
from json import load
from typing import List
from typing import Optional

from yaml import safe_load
from yaml.scanner import ScannerError

from . import description as all_descriptions
from .exception import DescriptionFileNotLoading
from .exception import FileHasNoExtension
from .exception import RequiredVariableNotSet


@dataclass
class VariableError:
    description_path: str | None = None
    errors: list[VariableError] = field(default_factory=list)


def load_env_file(file_path) -> dict:
    if not os.path.isfile(file_path):
        raise FileNotFoundError(f"{file_path} does not exist.")
    with open(file_path) as file:
        env_vars = {}
        for line in file:
            if not line.startswith("#"):
                key, value = line.strip().split("=")
                env_vars[key] = value
    return env_vars


def load_all_env_vars() -> dict:
    env_vars = {}
    for key, value in os.environ.items():
        env_vars[key] = value
    return env_vars


def load_json_file(file_path: str) -> dict:
    if not os.path.isfile(file_path):
        raise FileNotFoundError(f"{file_path} does not exist.")
    with open(file_path) as file:
        json_data = load(file)
    return json_data


def get_file_extension(file_path) -> str:
    _, file_extension = os.path.splitext(file_path)
    return file_extension.lower()


def load_yaml_file(file_path):
    if not os.path.isfile(file_path):
        raise FileNotFoundError(f"{file_path} does not exist.")
    with open(file_path) as yaml_file:
        data = safe_load(yaml_file)
    return data


def to_snake_case(name):
    name = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
    name = re.sub("__([A-Z])", r"_\1", name)
    name = re.sub("([a-z0-9])([A-Z])", r"\1_\2", name)
    return name.lower()


def is_valid_env(expected_env: dict, actual_env: dict) -> True | None:
    invalid_vars = []
    for key, values in expected_env.items():
        fails: list[str] = []
        for name, klass in inspect.getmembers(all_descriptions):
            klass_name = to_snake_case(name)
            if klass_name not in values:
                # user did not use this description
                continue
            description = klass(values[klass_name])
            value = actual_env.get(key, None)
            is_required = values.get("required", True)
            if is_required and value == None:
                raise RequiredVariableNotSet("is_required not but set")
            if value != None and not description.does_pass(value):
                fails.append(klass_name)
        if fails.__len__() > 0:
            invalid_vars.append([key, fails])
    invalid_vars.sort(key=lambda fail: fail[0])
    return invalid_vars if invalid_vars.__len__() > 0 else True


def get_errors_for(env: dict, descriptions: list[str]) -> list[VariableError]:
    errors: list[VariableError] = []
    for path in descriptions:
        extension = get_file_extension(path)
        try:
            if extension == ".json":
                description: dict = load_json_file(path)
            elif extension in [".yml", ".yaml"]:
                description: dict = load_yaml_file(path)
            else:
                raise FileHasNoExtension(
                    f"make sure the description file ends with: .json/.yaml/.yml"
                )
        except (
            FileNotFoundError,
            JSONDecodeError,
            ScannerError,
            ValueError,
        ) as exc:
            raise DescriptionFileNotLoading(
                f"couldn't load file at:{path}, {exc}")
        is_valid = is_valid_env(description, env)
        if is_valid != True:
            errors.append(VariableError(
                description_path=path, errors=is_valid))
    return errors



from __future__ import annotations

import setuptools

with open('README.md', encoding='utf-8') as fh:
    README = fh.read()

setuptools.setup(
    name='env_should_be',
    author='Wassef911',
    author_email='wassef911@gmail.com',
    description='a cli to help, build multi env applications.',
    keywords='python3, pypi, package',
    long_description_content_type='text/markdown',
    long_description=README,
    url='https://github.com/wassef911/env_should_be',
    package_dir={'': 'src'},
    packages=setuptools.find_packages(where='src'),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.11.4',
    extras_require={
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },
    entry_points={
        'console_scripts': [
            'env_should_be=env_should_be.main:main',
        ],
    },
)



from __future__ import annotations

import unittest

from src.env_should_be.utils import is_valid_env


class TestIsValidEnv(unittest.TestCase):
    def test_valid_env(self):
        expected_env = {
            'DB_USER': {'length': 6, 'regex': '^[a-zA-Z0-9]+$', 'required': True},
            'DB_PASSWORD': {
                'length': 11,
                'regex': r'^(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9])(?=.*[!@#\$%\^&\*])(?=.{8,})',
                'required': True,
            },
            'DB_HOST': {'option': ['localhost', '127.0.0.1'], 'required': True},
            'DB_PORT': {'length': 4, 'regex': '^[0-9]+$'},
            'APP_ENV': {'option': ['dev', 'prod'], 'required': True},
        }

        actual_env = {
            'DB_USER': 'myuser',
            'DB_PASSWORD': 'MyPassw0rd!',
            'DB_HOST': 'localhost',
            'DB_PORT': '3306',
            'APP_ENV': 'dev',
        }
        self.assertEqual(is_valid_env(expected_env, actual_env), True)

    def test_invalid_env(self):
        expected_env = {
            'DB_USER': {'length': 8, 'regex': '^[a-zA-Z0-9]+$', 'required': True},
            'DB_PASSWORD': {
                'length': 12,
                'regex': r'^(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9])(?=.*[!@#\$%\^&\*])(?=.{8,})',
                'required': True,
            },
            'DB_HOST': {'option': ['localhost', '127.0.0.1'], 'required': True},
            'DB_PORT': {'length': 4, 'regex': '^[0-9]+$'},
            'APP_ENV': {'option': ['dev', 'prod'], 'required': True},
        }

        actual_env = {
            'DB_USER': 'myuser',
            'DB_PASSWORD': 'MyPassw0rd',
            'DB_HOST': '127.0.0.2',
            'DB_PORT': '3306',
            'APP_ENV': 'testing',
        }
        self.assertEqual(
            is_valid_env(expected_env, actual_env),
            [
                ['APP_ENV', ['option']],
                ['DB_HOST', ['option']],
                ['DB_PASSWORD', ['length', 'regex']],
                ['DB_USER', ['length']],
            ],
        )

    def test_required_arg_for_env(self):
        expected_env = {
            'DB_USER': {
                'length': 6,
            },
            'DB_HOST': {
                'option': ['localhost'],
            },
            'APP_NAME': {
                'length': 6,
                'required': False,  # meaning it will only be validated if it exists
            },
            'SENT_ARTIFACT_EVERY_N_DAYS': {'is_int': True, 'is_float': False},
            'MAX_CURRENCY': {'is_int': False, 'is_float': True},
        }

        actual_env = {
            'DB_USER': 'myuser',
            'DB_HOST': 'localhost',
            'SENT_ARTIFACT_EVERY_N_DAYS': 5,
            'MAX_CURRENCY': 2.5,
        }
        self.assertEqual(
            is_valid_env(expected_env, actual_env),
            True,
        )

        self.assertEqual(
            is_valid_env(expected_env, {'APP_NAME': 'A', **actual_env}),
            [
                ['APP_NAME', ['length']],
            ],
        )

        self.assertEqual(
            is_valid_env(expected_env, {'APP_NAME': 2.5, **actual_env}),
            [
                ['APP_NAME', ['length']],
            ],
        )
        self.assertEqual(
            is_valid_env(expected_env, {'APP_NAME': 'AWERTY', **actual_env}),
            True,
        )


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

__all__ = (
    "Boolean",
    "Length",
    "MinLength",
    "MaxLength",
    "Regex",
    "Option",
    "Constant",
    "IsInt",
    "IsStr",
    "IsFloat",
)

import re
from abc import ABC
from abc import abstractmethod
from typing import Optional, Any, List
from .exception import ValueUnassignableToDescription


class Description(ABC):
    def __init__(self, value):
        self.value = value

    @abstractmethod
    def is_valid(self, value) -> bool:
        pass

    @property
    def value(self):
        return self.__value

    @value.setter
    def value(self, val):
        if self.is_valid(val):
            self.__value = val
            return
        raise ValueUnassignableToDescription(
            f"Value {val} is Invalid, can't be assigned to {self.__class__.__name__} "
        )

    @abstractmethod
    def does_pass(self, actual: str | None) -> bool:
        pass


class Boolean(Description):
    def is_valid(self, value):
        return isinstance(value, bool)


class Length(Description):
    def is_valid(self, value):
        return isinstance(value, int) and value > 0

    def does_pass(self, actual: str | None) -> bool:
        return (
            actual is not None
            and hasattr(actual, "__len__")
            and actual.__len__() == self.value
        )


class MinLength(Length):
    def does_pass(self, actual: str | None) -> bool:
        return (
            isinstance(actual, str)
            and hasattr(actual, "__len__")
            and actual.__len__() >= self.value
        )


class MaxLength(Length):
    def does_pass(self, actual: str | None) -> bool:
        return (
            isinstance(actual, str)
            and hasattr(actual, "__len__")
            and actual.__len__() <= self.value
        )


class Regex(Description):
    def is_valid(self, value):
        try:
            re.compile(value)
            return True
        except Exception:
            return False

    def does_pass(self, actual: str | None) -> bool:
        return actual != None and re.match(self.value, actual)


class Option(Description):
    def is_valid(self, value: list[Any]):
        return (
            isinstance(value, List)
            and hasattr(value, "__iter__")
            and value.__len__() > 0
        )

    def does_pass(self, actual: str | None) -> bool:
        return actual in self.value


class Constant(Description):
    def is_valid(self, value):
        try:
            x = str(value)
            return x.__len__() > 0  # can't use an empty string!
        except Exception:
            return False

    def does_pass(self, actual: str | None) -> bool:
        return str(actual) == str(self.value)


class IsInt(Boolean):
    def does_pass(self, actual):
        return isinstance(actual, int) == self.value


class IsStr(Boolean):
    def does_pass(self, actual):
        return isinstance(actual, str) == self.value


class IsFloat(Boolean):
    def does_pass(self, actual):
        return isinstance(actual, float) == self.value
